@@ -11,10 +11,10 @@
 #    under the License.
 
 import argparse
-import json
 import os
 
 from datetime import datetime
+from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 
 from karborclient.common.apiclient import exceptions
@@ -31,11 +31,6 @@ from karborclient import utils as arg_utils
            const=1,
            default=0,
            help='Shows details for all tenants. Admin only.')
-@utils.arg('--all_tenants',
-           nargs='?',
-           type=int,
-           const=1,
-           help=argparse.SUPPRESS)
 @utils.arg('--name',
            metavar='<name>',
            default=None,
@@ -188,6 +183,8 @@ def do_plan_delete(cs, args):
            help="Id of plan to update.")
 @utils.arg("--name", metavar="<name>",
            help="A name to which the plan will be renamed.")
+@utils.arg("--description", metavar="<description>",
+           help="Description to which the plan will be updated.")
 @utils.arg("--resources", metavar="<id=type=name,id=type=name>",
            help="Resources to which the plan will be updated.")
 @utils.arg("--status", metavar="<suspended|started>",
@@ -197,6 +194,8 @@ def do_plan_update(cs, args):
     data = {}
     if args.name is not None:
         data['name'] = args.name
+    if args.description is not None:
+            data['description'] = args.description
     if args.resources is not None:
         plan_resources = arg_utils.extract_resources(args)
         data['resources'] = plan_resources
@@ -282,11 +281,6 @@ def do_restore_create(cs, args):
            const=1,
            default=0,
            help='Shows details for all tenants. Admin only.')
-@utils.arg('--all_tenants',
-           nargs='?',
-           type=int,
-           const=1,
-           help=argparse.SUPPRESS)
 @utils.arg('--status',
            metavar='<status>',
            default=None,
@@ -349,7 +343,7 @@ def do_restore_list(cs, args):
         sortby_index = None
     else:
         sortby_index = 0
-    formatters = {"Parameters": lambda obj: json.dumps(
+    formatters = {"Parameters": lambda obj: jsonutils.dumps(
         obj.parameters, indent=2, sort_keys=True)}
     utils.print_list(restores, key_list, exclude_unavailable=True,
                      sortby_index=sortby_index, formatters=formatters)
@@ -412,11 +406,6 @@ def do_verification_create(cs, args):
            const=1,
            default=0,
            help='Shows details for all tenants. Admin only.')
-@utils.arg('--all_tenants',
-           nargs='?',
-           type=int,
-           const=1,
-           help=argparse.SUPPRESS)
 @utils.arg('--status',
            metavar='<status>',
            default=None,
@@ -482,7 +471,7 @@ def do_verification_list(cs, args):
         sortby_index = None
     else:
         sortby_index = 0
-    formatters = {"Parameters": lambda obj: json.dumps(
+    formatters = {"Parameters": lambda obj: jsonutils.dumps(
         obj.parameters, indent=2, sort_keys=True)}
     utils.print_list(verifications, key_list, exclude_unavailable=True,
                      sortby_index=sortby_index, formatters=formatters)
@@ -610,7 +599,7 @@ def do_protectable_list_instances(cs, args):
     else:
         sortby_index = 0
 
-    formatters = {"Dependent resources": lambda obj: json.dumps(
+    formatters = {"Dependent resources": lambda obj: jsonutils.dumps(
         obj.dependent_resources, indent=2, sort_keys=True)}
     utils.print_list(instances, key_list, exclude_unavailable=True,
                      sortby_index=sortby_index, formatters=formatters)
@@ -712,6 +701,14 @@ def do_checkpoint_create(cs, args):
                      json_format_list=json_format_list)
 
 
+@utils.arg('--all-tenants',
+           dest='all_tenants',
+           metavar='<0|1>',
+           nargs='?',
+           type=int,
+           const=1,
+           default=0,
+           help='Shows details for all tenants. Admin only.')
 @utils.arg('provider_id',
            metavar='<provider_id>',
            help='ID of provider.')
@@ -785,6 +782,7 @@ def do_checkpoint_list(cs, args):
         'start_date': args.start_date,
         'end_date': args.end_date,
         'project_id': args.project_id,
+        'all_tenants': args.all_tenants
     }
 
     if args.sort and (args.sort_key or args.sort_dir):
@@ -852,6 +850,45 @@ def do_checkpoint_delete(cs, args):
                                       "specified checkpoint.")
 
 
+@utils.arg('provider_id',
+           metavar='<provider_id>',
+           help='Id of provider.')
+@utils.arg('checkpoint',
+           metavar='<checkpoint>',
+           nargs="+",
+           help='ID of checkpoint.')
+@utils.arg('--available',
+           action='store_const',
+           dest='state',
+           default='error',
+           const='available',
+           help='Request the checkpoint be reset to "available" state instead '
+                'of "error" state(the default).')
+def do_checkpoint_reset_state(cs, args):
+    """Reset state of a checkpoint."""
+    failure_count = 0
+
+    for checkpoint_id in args.checkpoint:
+        try:
+            cs.checkpoints.reset_state(args.provider_id, checkpoint_id,
+                                       args.state)
+        except exceptions.NotFound:
+            failure_count += 1
+            print("Failed to reset state of '{0}'; checkpoint not found".
+                  format(checkpoint_id))
+        except exceptions.Forbidden:
+            failure_count += 1
+            print("Failed to reset state of '{0}'; not allowed".
+                  format(checkpoint_id))
+        except exceptions.BadRequest:
+            failure_count += 1
+            print("Failed to reset state of '{0}'; invalid input or "
+                  "current checkpoint state".format(checkpoint_id))
+    if failure_count == len(args.checkpoint):
+        raise exceptions.CommandError("Unable to find or reset any of the "
+                                      "specified checkpoint's state.")
+
+
 @utils.arg('--all-tenants',
            dest='all_tenants',
            metavar='<0|1>',
@@ -860,11 +897,6 @@ def do_checkpoint_delete(cs, args):
            const=1,
            default=0,
            help='Shows details for all tenants. Admin only.')
-@utils.arg('--all_tenants',
-           nargs='?',
-           type=int,
-           const=1,
-           help=argparse.SUPPRESS)
 @utils.arg('--name',
            metavar='<name>',
            default=None,
@@ -937,7 +969,7 @@ def do_trigger_list(cs, args):
     else:
         sortby_index = 0
 
-    formatters = {"Properties": lambda obj: json.dumps(
+    formatters = {"Properties": lambda obj: jsonutils.dumps(
         obj.properties, indent=2, sort_keys=True)}
     utils.print_list(triggers, key_list, exclude_unavailable=True,
                      sortby_index=sortby_index, formatters=formatters)
@@ -970,7 +1002,8 @@ def do_trigger_update(cs, args):
     """Update a trigger."""
     trigger_info = {}
     trigger_properties = arg_utils.extract_properties(args)
-    trigger_info['name'] = args.name
+    if args.name:
+        trigger_info['name'] = args.name
     trigger_info['properties'] = trigger_properties
     trigger = cs.triggers.update(args.trigger_id, trigger_info)
     dict_format_list = {"properties"}
@@ -1015,11 +1048,6 @@ def do_trigger_delete(cs, args):
            const=1,
            default=0,
            help='Shows details for all tenants. Admin only.')
-@utils.arg('--all_tenants',
-           nargs='?',
-           type=int,
-           const=1,
-           help=argparse.SUPPRESS)
 @utils.arg('--name',
            metavar='<name>',
            default=None,
@@ -1106,7 +1134,9 @@ def do_scheduledoperation_list(cs, args):
            help='Trigger name.')
 @utils.arg('operation_type',
            metavar='<operation_type>',
-           help='Operation Type of scheduled operation.')
+           choices=['protect', 'retention_protect'],
+           help='Operation Type of scheduled operation. Valid values are '
+                '"protect" or "retention_protect."')
 @utils.arg('trigger_id',
            metavar='<trigger_id>',
            help='Trigger id of scheduled operation.')
@@ -1165,11 +1195,6 @@ def do_scheduledoperation_delete(cs, args):
            const=1,
            default=0,
            help='Shows details for all tenants. Admin only.')
-@utils.arg('--all_tenants',
-           nargs='?',
-           type=int,
-           const=1,
-           help=argparse.SUPPRESS)
 @utils.arg('--status',
            metavar='<status>',
            default=None,
